CanvasRenderingContext2D

Struct CanvasRenderingContext2D 

#[repr(C)]
pub(crate) struct CanvasRenderingContext2D { reflector_: Reflector<AssociatedMemory>, canvas: HTMLCanvasElementOrOffscreenCanvas, canvas_state: CanvasState, }

Fields

reflector_: Reflector<AssociatedMemory>canvas: HTMLCanvasElementOrOffscreenCanvascanvas_state: CanvasState

Implementations

Source

impl CanvasRenderingContext2D

Source

impl CanvasRenderingContext2D

const RGBA8_BYTES_PER_PIXEL: usize = 4

const ASSOCIATED_MEMORY_BUFFER_COUNT: usize = 4

Ideally this would be two bitmap buffers, unfortunately we currently have ~4 copies which are all retained until GC:

  1. The draw target’s backing bitmap in the canvas paint thread (CanvasData::draw_target, e.g. pixmap in VelloCPUDrawTarget).
  2. Additional backend resources (approximated with 1 bitmap buffer, actually depends on backend): e.g. ctx and resources in VelloCPUDrawTarget for vello_cpu. TODO: #46785 tracks getting accurate statistics from the backend.
  3. CachedImageData::Raw in WebRender (cpu memory copy).
  4. GPU memory in webrender / driver (also lives in RAM on unified memory systems).

This metric is only used to inform the GC about memory pressure, so it doesn’t need to be completely accurate, but undercounting may delay GCs and hence increase memory usage. On mobile devices this slightly undercounts (usually unified memory), on desktop this slightly overcounts, since GPU memory is separate.

fn associated_memory_size(size: Size2D<u64>) -> usize

pub(crate) fn update_associated_memory_size(&self)

pub(crate) fn new_inherited( global: &GlobalScope, canvas: HTMLCanvasElementOrOffscreenCanvas, size: Size2D<u32>, ) -> Option<CanvasRenderingContext2D>

pub(crate) fn new( cx: &mut JSContext, global: &GlobalScope, canvas: &HTMLCanvasElement, size: Size2D<u32>, ) -> Option<DomRoot<CanvasRenderingContext2D>>

pub(crate) fn send_canvas_command_immediate(&self, msg: CanvasCommand)

pub(crate) fn set_image_key(&self, image_key: ImageKey)

pub(crate) fn update_rendering(&self, canvas_epoch: Epoch) -> bool